package tallerVehiculos;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;

import javax.swing.JOptionPane;

import paneles.*;

public class Controlador {
	
	//Datos Modelo
	private GestionUsuario gestorUsuarios;
	//Datos Vista
	private Taller taller; //JFrame principal
	//Paneles para la Vista
	private UsuarioLogin panelLogin;
	
	public Controlador (Taller t) {
		taller = t;
		iniciarGestorUsuarios();
		panelLogin = new UsuarioLogin();
		asignarAcciones();
		mostrarPanelLogin();
	}
	
	private void asignarAcciones() {
		panelLogin.botonCrear.addActionListener(new AccionCrearUsuario());
	}
	
	/**
	 * Establece en el marco principal que muestre
	 * el panel de inicio de sesion de usuarios.
	 */
	public void mostrarPanelLogin() {
		panelLogin.reset();
		taller.setContentPane(panelLogin);
		taller.pack();
		taller.setLocationRelativeTo(null);
	}
	
	/**
	 * Intentará recuperar los datos de gestion de
	 * usuarios guardados en disco.<br>Si no puede
	 * recuperarlos, ya sea porque no existe el archivo
	 * o porque no tiene acceso a él o está corrupto,
	 * iniciará un Gestor de Usuarios nuevo.
	 */
	private void iniciarGestorUsuarios() {
		File usuarios = new File("usuarios.bin");
		if (usuarios.exists()) {
			ObjectInputStream ois;
			try {
				ois = new ObjectInputStream(new FileInputStream(usuarios));
				Object datos = ois.readObject();
				if (datos instanceof GestionUsuario) {
					gestorUsuarios = (GestionUsuario)datos;
				}
				else {
					mensajeEmergente("Datos de usuarios guardados no son válidos", "Gestion Usuarios");
					gestorUsuarios = new GestionUsuario();
				}
				ois.close();
			} catch (IOException e) {
				mensajeEmergente("No se pudo acceder a:\n" + usuarios.getAbsolutePath(), "Gestion Usuarios");
				gestorUsuarios = new GestionUsuario();
			} catch (ClassNotFoundException e) {
				mensajeEmergente("Datos de usuarios guardados no son válidos", "Gestion Usuarios");
				gestorUsuarios = new GestionUsuario();
			}
			
		}
		else {
			mensajeEmergente("No hay datos de Usuarios guardados", "Gestion Usuarios");
			gestorUsuarios = new GestionUsuario();
		}
	}
	
	// ++++++++++++++++++++ Clases Action Listener  ++++++++++++++++++++
	class AccionCrearUsuario implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			if (panelLogin.campoNombre.getText().isEmpty() || panelLogin.campoPassword.getPassword().length == 0)
				mensajeEmergente("Ha de rellenar los dos campos", "Crear Usuario");
			else {
				String nombre = panelLogin.campoNombre.getText();
				String password = String.copyValueOf(panelLogin.campoPassword.getPassword());
				String confirm = JOptionPane.showInputDialog(null, "Por favor, confirme el Password:");
				if (password.equals(confirm)) {
					if (gestorUsuarios.nuevoUsuario(new Usuario(nombre, password))) {
						mensajeEmergente("Nuevo usuario creado.\nSesión iniciada",
								"Crear Usuario");
						gestorUsuarios.guardarUsuarios();
						taller.usuarioLogueado = true;
						taller.setContentPane(new PanelBienvenida(nombre));
						taller.pack();
						taller.setLocationRelativeTo(null);
					}
				}
				else
					mensajeEmergente("El password de registro no coincide", "Crear Usuario");
			}
			
		}
	}
	
	/**
	 * Puesto que en este código se va a recurrir varias veces a mostrar
	 * JOptionPane informativos en pantalla, vale la pena simplificar el código
	 * creando este método para no repetirlo decenas de veces
	 * @param mensaje Texto informativo que mostrará el JOptionPane
	 * @param titulo Título descriptivo del JOptionPane
	 */
	private void mensajeEmergente(String mensaje, String titulo) {
		JOptionPane.showMessageDialog(null, mensaje, titulo, JOptionPane.WARNING_MESSAGE);
	}

}
